import os, yaml

# Channels are expected to be named <major>.<minor>-{candidate,fast,release}
PACKAGE_FILE_FORMAT = {
    'channels': [
            {'name': '{}-candidate',
            'currentCSV': ''},
            {'name': '{}-fast',
            'currentCSV': ''},
            {'name': '{}-stable',
            'currentCSV': ''}
    ],
    'defaultChannel': '{}-{}',
    'packageName': ''
}

class Package:
    PACKAGE_FILE_FORMAT = {
        'channels': [],
        'defaultChannel': '{}-{}',
        'packageName': ''
    }

    def __init__(self, package=None, operator=None, default_channel=None):
        if package is not None:
            self.operator = package['packageName']
            self.channels = []
            self.default_channel = package['defaultChannel']
            for c in package['channels']:
                ch = Channel(c['name'], c['currentCSV'])
                self.channels.append(ch)
        else:
            self.channels = []
            self.operator = operator if operator is not None else ''
            self.default_channel = default_channel if default_channel is not None else ''

    def __str__(self):
        return 'Package class for {} operator'.format(self.operator)

    def get_channel(self, name):
        for c in self.channels:
            if c.get_name() == name:
                return c

    def get_channels(self):
        channels = []
        for c in self.channels:
            channels.append(c)
        return channels

    def create_channel(self, name, current_csv, default=False):
        ch = Channel(name, current_csv)
        self.channels.append(ch)
        if default:
            self.default_channel = name

    def get_formatted(self):
        f = self.PACKAGE_FILE_FORMAT
        f['packageName'] = self.operator
        f['defaultChannel'] = self.default_channel
        f['channels'] = []
        for c in self.channels:
            f['channels'].append({'name': c.get_name(), 'currentCSV': c.get_current_csv()})
        return f

    def set_default_channel(self, default_channel):
        self.default_channel = default_channel

    def set_operator(self, operator):
        self.operator = operator

    def update_channel(self, channel, new_csv):
        for c in self.channels:
            if c.get_name() == channel:
                c.set_current_csv(new_csv)

    def promote_channel(self, promote_from, promote_to):
        self.update_channel(promote_to, self.get_channel(promote_from).get_current_csv())

class Channel:
    def __init__(self, name, currentCSV):
        self.name = name
        self.currentCSV = currentCSV

    def get_current_csv(self):
        return self.currentCSV

    def set_current_csv(self, new_csv):
        self.currentCSV = new_csv

    def get_name(self):
        return self.name

    def set_name(self, name):
        self.name = name


# Defines what an operator image definition looks like
class Operatorimage:
    deployment = None
    container  = None
    image      = None
    digest     = None
    tag        = None
    image_name = None
    image_repo = None

    def __init__(self, deployment=None, container=None, image=None):
        self.deployment = deployment
        self.container = container
        self.image = image
        # Everything before the last '/' should make up repo
        self.image_repo = '/'.join(image.split('/')[:-1])
        remainder = image.split('/')[-1]
        if '@' in remainder:
            self.image_name = remainder.split('@')[0]
            self.digest     = remainder.split('@')[1]
        else:
            self.image_name = remainder.split(':')[0]
            self.tag        = remainder.split(':')[1]
    
    def set_digest(self, digest):
        self.digest = digest
        # We're assuming that when we're called the existing image doesn't have a digest
        self.image = '{}@{}'.format(self.image.split(':')[0], digest)


from .images import Image
from artifactory import ArtifactoryPath
import os, sys, json
import requests

class ImageRepo:
    """ This is a class to provide a general interface to query container image registry servers.
        Hacked together in a hurry, will probably be refactored to be more pythonic - if there's a better way to do this
    """
    def __init__(self, image, logger=None):
        self.image = image

        # Check for some well known repos
        if 'artifactory' in self.image.get_image_repo():
            self.image_repo = ArtifactoryRepo(self.image)
        elif self.image.get_image_repo().startswith('quay.io/'):
            self.image_repo = QuayRepo(self.image)
        elif self.image.get_image_repo().startswith('docker.io'):
            self.image_repo = DockerRepo(self.image)

        else:
            raise RepoTypeNotImplemented('Unknown repository type for image {}'.format(image.get_image()))

    def get_manifest_list_digest(self):
        return self.image_repo.get_manifest_list_digest()

    def get_image_digest(self):
        return self.image_repo.get_image_digest()

    def get_raw_manifest_list(self):
        """Return the docker manifest list in json format

        :raises ManifestListNotFound: description

        :return: manifest.list.json content
        :rtype: dict
        """
        return self.image_repo.get_raw_manifest_list()

class ArtifactoryRepo:
    # Allow credentials to be shared between instances
    _artifactory_user = None
    _artifactory_key = None
    _artifactory_base = None

    def __init__(self, image, artifactory_base=None, artifactory_user=None, artifactory_key=None, logger=None):
        self.image = image

        # Always give provided credential preference
        if artifactory_user:
            self.artifactory_user = artifactory_user
        elif self._artifactory_user:
            self.artifactory_user = self._artifactory_user
        elif 'ARTIFACTORY_USER' in os.environ:
            # Go to environment variables if we don't have them as class variable either
            self._artifactory_user = os.getenv('ARTIFACTORY_USER')
            self.artifactory_user = self._artifactory_user
        else:
            # This is where we should panic and throw some orderly exception
            raise MissingCredentials("No artifactory user provided or found in ARTIFACTORY_USER environment variable")

        if artifactory_key:
            self.artifactory_key = artifactory_key
        elif self._artifactory_key:
            self.artifactory_key = self._artifactory_key
        elif 'ARTIFACTORY_KEY' in os.environ:
            self._artifactory_key = os.getenv('ARTIFACTORY_KEY')
            self.artifactory_key = self._artifactory_key
        else:
            # This is where we should panic and throw some orderly exception
            raise MissingCredentials("No artifactory key provided or found in ARTIFACTORY_KEY environment variable")

        if artifactory_base:
            self.artifactory_base = artifactory_base
        elif self._artifactory_base:
            self.artifactory_base = self._artifactory_base
        elif 'ARTIFACTORY_BASE' in os.environ:
            self._artifactory_base = os.getenv('ARTIFACTORY_BASE')
            self.artifactory_base = self._artifactory_base
        else:
            # This is where we should panic and throw some orderly exception
            raise MissingCredentials("No artifactory base provided or found in ARTIFACTORY_BASE environment variable")

    def get_image_digest(self):
        # We know we're always querying for sha256
        return 'sha256:{}'.format(self._get_raw_image_digest())

    def _get_artifactory_repo(self):
        # For artifactory we need to massage the repo string a bit
        ### Split out all directories after artifactory.com/
        p = '/'.join(self.image.get_image_repo().split('/')[1:])
        ### Split out the first part of repo.artifactory.com
        r = self.image.get_image_repo().split('.')[0]
        return '{}/{}'.format(r, p)

    def _get_raw_image_digest(self):
        manifestpath = '/'.join([
                        self.artifactory_base,
                        self._get_artifactory_repo(), # We have to massage the repo for artifactory
                        self.image.get_image_name(),
                        self.image.get_tag(),
                        "manifest.json"
                    ])
        manifest_path = ArtifactoryPath(manifestpath, auth=(self.artifactory_user, self.artifactory_key))

        try:
            return ArtifactoryPath.stat(manifest_path).sha256
        except FileNotFoundError as e:
            raise ManifestNotFound(e)

    def get_manifest_list_digest(self):
        # We know we're always querying for sha256
        return 'sha256:{}'.format(self._get_raw_manifest_list_digest())

    def _get_raw_manifest_list_digest(self):
        listpath = '/'.join([
                        self.artifactory_base,
                        self._get_artifactory_repo(), # We have to massage the repo for artifactory
                        self.image.get_image_name(),
                        self.image.get_tag(),
                        "list.manifest.json"
                    ])
        list_path = ArtifactoryPath(listpath, auth=(self.artifactory_user, self.artifactory_key))

        try:
            return ArtifactoryPath.stat(list_path).sha256
        except FileNotFoundError as e:
            raise ManifestListNotFound(e)

    def get_raw_manifest_list(self):
        """Return the docker manifest list in json format

        :raises ManifestListNotFound: description

        :return: manifest.list.json content
        :rtype: dict
        """

        listpath = '/'.join([
                        self.artifactory_base,
                        self._get_artifactory_repo(), # We have to massage the repo for artifactory
                        self.image.get_image_name(),
                        self.image.get_tag(),
                        "list.manifest.json"
                    ])
        list_path = ArtifactoryPath(listpath, auth=(self.artifactory_user, self.artifactory_key))

        try:
            f = list_path.open()
        except FileNotFoundError as e:
            raise ManifestListNotFound(e)
        except RuntimeError as e:
            raise ManifestListNotFound(e)
        return json.loads(f.read().decode('utf-8'))

class QuayRepo:
    QUAY_BASE_URL = 'https://quay.io/api/v1/repository'

    def __init__(self, image):
        self.image = image

    def get_image_digest(self):
        return self._get_digest(manifest_list=False)

    def get_manifest_list_digest(self):
        return self._get_digest(manifest_list=True)

    def _get_digest(self, manifest_list):
        url = '/'.join([
                        self.QUAY_BASE_URL,
                        self._get_quay_repo(),
                        'tag',
                        '?onlyActiveTags=true&specificTag='
                ])

        resp = requests.get(url + self.image.get_tag())

        if resp.status_code == 403:
            raise MissingCredentials(resp.text)
        elif resp.status_code == 404:
            if manifest_list:
                raise ManifestListNotFound(resp.text)
            else:
                raise ManifestNotFound(resp.text)
        elif not resp.status_code == 200:
            raise Exception(resp.text)

        # Since we query for specific tag we expect single response
        tags = resp.json()['tags']
        if len(tags) > 1:
            raise Exception('Expected 1 tag, found {}. {}'.format(len(tags), tags))
        for t in tags:
            if t['is_manifest_list'] == manifest_list:
                return t['manifest_digest']
            else:
                if manifest_list:
                    raise ManifestListNotFound('Tag {} is not manifest list'.format(self.image.get_tag()))
                else:
                    raise ManifestNotFound('Tag {} is a manifest list'.format(self.image.get_tag()))

    def _get_quay_repo(self):
        r = self.image.get_image_repo().replace('quay.io/','')
        return '/'.join([r, self.image.get_image_name()])

class DockerRepo:
    """ This class provides an interface for docker hub images allowing one to query image and manifest list digests
        as well as get the raw manifest list in json format.
    """

    def __init__(self,image):
        self.image = image
        self.org = image.get_image().split('/')[1]
        self.repo = image.get_image_name()
        self.tag = image.get_tag()

    def get_image_digest(self):
        return self._get_digest(manifest_list=False)

    def get_manifest_list_digest(self):
        return self._get_digest(manifest_list=True)

    def get_raw_manifest_list(self):
        """ Return the docker manifest list in json format
        
        :raises ManifestListNotFound: No manifest list exists for the specified image.

        :returns: manifest.list.json content
        :rtype: dict
        """
        ## Get token
        t=requests.get('https://auth.docker.io/token?scope=repository%3A{org}%2F{repo}%3Apull&service=registry.docker.io'.format(org=self.org, repo=self.repo))
        token=t.json()['token']

        ## check media type
        headers={'accept':'application/vnd.docker.distribution.manifest.list.v2+json', 'Authorization': 'Bearer {}'.format(token)}
        m=requests.get('https://registry-1.docker.io/v2/{org}/{repo}/manifests/{tag}'.format(org=self.org,repo=self.repo,tag=self.tag), headers=headers)
        
        if 'manifest.list' in m.headers['Content-Type']:
            return m.json()
        else:
            raise ManifestListNotFound('No manifest for: ' + self.image.get_image())

    def _get_digest(self, manifest_list):
        """ Return the digest of the docker image or manifest list

        :raises ManifestListNotFound: if manifest list is requested but it does not exist

        :return: docker manifest/manifest list digest
        :rtype: string
        """
        ## Get token
        t=requests.get('https://auth.docker.io/token?scope=repository%3A{org}%2F{repo}%3Apull&service=registry.docker.io'.format(org=self.org, repo=self.repo))
        token=t.json()['token']

        ## check media type
        headers={'accept':'application/vnd.docker.distribution.manifest.list.v2+json', 'Authorization': 'Bearer {}'.format(token)}
        m=requests.get('https://registry-1.docker.io/v2/{org}/{repo}/manifests/{tag}'.format(org=self.org,repo=self.repo,tag=self.tag), headers=headers)

        if 'manifest.list' in m.headers['Content-Type']:
            if manifest_list:
                return m.headers['Docker-Content-Digest']
        else:
            if manifest_list:
                raise ManifestListNotFound("Manifest List does not exist")
            else:
                ## Get the proper digest for single arch image - need the correct header
                headers={'accept':'application/vnd.docker.distribution.manifest.v2+json', 'Authorization': 'Bearer {}'.format(token)}
                m=requests.get('https://registry-1.docker.io/v2/{org}/{repo}/manifests/{tag}'.format(org=self.org,repo=self.repo,tag=self.tag), headers=headers)
                return m.headers['Docker-Content-Digest']

class MissingCredentials(Exception):
    pass

class RepoTypeNotImplemented(Exception):
    pass

class ManifestListNotFound(Exception):
    pass

class ManifestNotFound(Exception):
    pass


import logging, sys, copy, yaml
from .images import Image

class _literal(str):
    pass

def _literal_presenter(dumper, data):
    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')

class ClusterServiceVersion:
    LATEST_IMAGE_INDICATOR   = '-latest'
    RELATED_IMAGE_IDENTIFIER = 'olm.relatedImage.'
    TAGGED_RELATED_IMAGE_IDENTIFIER = 'olm.tag.relatedImage.'

    def __init__(self, csv, name=None, target_version=None, replaces=None, skiprange=None, logger=None):
        self.original_csv = csv
        self.csv = copy.deepcopy(self.original_csv)

        self.original_operator_images    = []
        self.operator_images             = []
        self.annotation_related_images   = []
        self.spec_related_images         = []

        # Holds the version information
        self.version = ''
        self.major_minor = ''
        self.versioned_name = ''
        self.replaces = None
        self.skiprange = None
        self.replaces = replaces

        # If name is not provided, we can try extrapolate it
        if name:
            self.name = name
        else:
            self.name = self.csv['metadata']['name'].split('.')[0]

        if skiprange:
            self.skiprange = skiprange

        if target_version:
            self.set_version(target_version)

        if logger:
            self.log = logger
        else:
            self._setup_basic_logger()

        # Extract some other useful info
        self._manipulate_tag_images()
        self._get_operator_images()
        self._get_related_images()

    def set_deployments_annotations(self, key=None, value=None):
        """Set key with value passed in for each deployment in the CSV

        :param key: Key being search for in deployment annotations
        :type key: string

        :param value: Value that will be assigned to the key passed in
        :type value: string
        """
        for d in self.csv['spec']['install']['spec']['deployments']:
            if not 'annotations' in d['spec']['template']['metadata']:
                continue
            d['spec']['template']['metadata']['annotations'][key] = value

    def set_container_image_annotation(self, image):
        """ Set metadata.annotations.containerImage with Image.image passed in

        :param image: Image that will be assigned to metadata.annotations.containerImage
        :type image: Image
        """
        self.csv['metadata']['annotations']['containerImage'] = image.image

    def set_version(self, version):
        """Set the target version for the CSV

        :param version: Target version in semver format X.Y.Z with optional -nnnn for pre-release
        :type version: string
        """
        self.version = version
        self.versioned_name = '{}.v{}'.format(self.name, self.version)
        self._update_version_references()

    def set_replaces(self, replaces):
        """ Set the release that this replaces

        :param replaces: The versioned name of the csv that this csv replaces
        :type replaces: string
        """
        self.replaces = replaces
        self.csv['spec']['replaces'] = self.replaces

    def set_image_pullsecret(self, name):
        """ Set the image pull secret for all operator deployment deployments. Overwrites any existing pull secret

        :param name: String or list of strings containing name of the pull secret to add
        :type name: string, list
        """
        if isinstance(name, str):
            p = [{'name': name}]
        else:
            p = [ {'name': x} for x in name ]

        for d in self.csv['spec']['install']['spec']['deployments']:
            d['spec']['template']['spec']['imagePullSecrets'] = p

    def add_image_pullsecret(self, name):
        """ Add image pull secret for all operator deployment deployments. Existing pull secrets will be kept

        :param name: String or list of strings containing name of the pull secret to add
        :type name: string, list
        """
        if isinstance(name, str):
            p = [{'name': name}]
        else:
            p = [ {'name': x} for x in name ]

        for d in self.csv['spec']['install']['spec']['deployments']:
            if not 'imagePullSecrets' in d['spec']['template']['spec']:
                # If imagepullsecret is missing, set it
                d['spec']['template']['spec']['imagePullSecrets'] = p
            else:
                # If imagepullsecret exists, add to the list
                if not(all(x in p for x in d['spec']['template']['spec']['imagePullSecrets'])):
                    try:
                        d['spec']['template']['spec']['imagePullSecrets'].extend(p)
                    except TypeError:
                        print('imagePullSecrets is not of type list')


    def generate_spec_relatedImages(self):
        """ Generates spec.relatedImages based on information found in operator deployment annotations marked with 'olm.relatedImage.*'
        """
        self.spec_related_images=self.annotation_related_images
        if 'relatedImages' in self.csv['spec']:
            self.log.debug('Resetting existing spec.relatedImages')
        self.csv['spec']['relatedImages'] = []

        # Hold dict of all names and images so we can find conflicting information
        images_validation = {}
        for r in self.spec_related_images:
            if r.name in images_validation:
                self.log.debug('found previous entry for {}'.format(r.name))
                if r.image != images_validation[r.name]:
                    self.log.warning('Validation error: Found different values for {}: {} and {}'.format(r.name, r.image, images_validation[r.name]))
                    self.log.debug('overwriting')
                    self.csv['spec']['relatedImages'][r.name] = r.image
                    continue

            self.csv['spec']['relatedImages'].append({
                'name':     r.name,
                'image':    r.image
            })

    def get_owned_crds(self):
        """ Returns a list of owned CustomResourceDefinitions

        :return: List of owned custom resource definitions as dict object
        :rtype: list
        """
        return self.csv['spec']['customresourcedefinitions']['owned']

    def get_updated_csv(self):
        """ Returns the updated CSV object

        :return: CSV with updated version and image information
        :rtype: dict
        """
        # Merge in the updates that are done to Operatorimages and Operandimages
        self._update_operator_container_images()
        self._update_operand_images()

        return self.csv

    def get_formatted_csv(self):
        """ Returns a stringified save ready formatted ClusterServiceVersion
            This allows maintaining the format of the `alm-examples: |-` block
        """
        yaml.add_representer(_literal, _literal_presenter)
        formatted_csv = self.get_updated_csv()
        formatted_csv['metadata']['annotations']['alm-examples'] = _literal(formatted_csv['metadata']['annotations']['alm-examples'])
        return yaml.dump(formatted_csv, default_flow_style=False)

    def get_replaces(self):
        """ Return String

        :return: Returns the csv file name with previous version
        :rtype: string
        """
        return self.csv['spec']['replaces']

    def get_operator_images(self):
        """ Return a list of images used for operator deployment

        :return: Returns a List of Images as defined in Images class
        :rtype: list
        """
        return self.operator_images

    def get_annotation_related_images(self):
        """ Return a list of images found in 'olm.relatedImages.*' deployment annotations

        :return: Returns a List of Images as defined in Images class
        :rtype: list
        """
        return self.annotation_related_images

    def get_operator_deployments(self, api_version='apps/v1', kind='Deployment'):
        """ Return a list of kubernetes deployment objects constructed from the CSV deployments section

        :return: List of kubernetes deployment objects
        :rtype: list
        """
        # Extract the deployment(s)
        deployments = []
        for d in self.csv['spec']['install']['spec']['deployments']:
            deployments.append(copy.deepcopy(d))

        # Adjust the dict to make it valid deployment object
        for d in deployments:
            d.update({"apiVersion": api_version, "kind": kind})
            d['metadata'] = {
                'name': d['name']
                }
            del(d['name'])

        return deployments

    def _update_version_references(self):
        """ Update the version specifc fields based on self.version
        """
        self.csv['spec']['version'] = self.version
        self.csv['metadata']['name'] = self.versioned_name
        if len(self.versioned_name) > 63:
            if hasattr(self, 'log'):
                # Fixing a weird issue where the logger is not available when we are constructing a csv object. I even tried switching the
                # order of parameters when creating the csv object in create-release.py
                self.log.warning('{} is longer than 63 characters, and may lead to problems'.format(self.name))
        if self.skiprange:
            self.csv['metadata']['annotations']['olm.skipRange'] = self.skiprange
        if self.replaces:
            self.csv['spec']['replaces'] = self.replaces
        else:
            if 'replaces' in self.csv['spec']:
                del(self.csv['spec']['replaces'])

    def _get_operator_images(self):
        """[Populate a list of all images that are used for the operator deployment]
        """
        for d in self.csv['spec']['install']['spec']['deployments']:
            for c in d['spec']['template']['spec']['containers']:
                o = Image(
                    deployment = d['name'],
                    container  = c['name'],
                    image      = c['image']
                )
                self.original_operator_images.append(o)
                self.operator_images.append(o)

    def _get_related_images(self):
        # Capture related images from annotations
        for d in self.csv['spec']['install']['spec']['deployments']:
            if not 'annotations' in d['spec']['template']['metadata']:
                continue
            for a in d['spec']['template']['metadata']['annotations']:
                if a.startswith(self.RELATED_IMAGE_IDENTIFIER):
                    o = Image(
                        deployment  = d['name'],
                        name        = a.replace(self.RELATED_IMAGE_IDENTIFIER, ''),
                        image       = d['spec']['template']['metadata']['annotations'][a]
                    )
                    self.annotation_related_images.append(o)

    def _manipulate_tag_images(self):
        taggedImages = {}
        for d in self.csv['spec']['install']['spec']['deployments']:
            if not 'annotations' in d['spec']['template']['metadata']:
                continue
            for a in d['spec']['template']['metadata']['annotations']:
                related_image_annotation = a.replace(self.TAGGED_RELATED_IMAGE_IDENTIFIER, self.RELATED_IMAGE_IDENTIFIER)
                # olm.relatedImage should take precedence over olm.tag.relatedImage so do not overwrite olm.relatedImage
                if a.startswith(self.TAGGED_RELATED_IMAGE_IDENTIFIER) and not related_image_annotation in d['spec']['template']['metadata']['annotations']:
                        taggedImages[related_image_annotation] = d['spec']['template']['metadata']['annotations'][a]
        for d in self.csv['spec']['install']['spec']['deployments']:
            if not 'annotations' in d['spec']['template']['metadata']:
                continue
            for i in taggedImages:
                d['spec']['template']['metadata']['annotations'][i] = taggedImages[i]

    def _update_operator_container_images(self):
        for image in self.operator_images:
            for d in self.csv['spec']['install']['spec']['deployments']:
                if d['name'] == image.deployment:
                    for c in d['spec']['template']['spec']['containers']:
                        if c['name'] == image.container:
                            c['image'] = image.image

    def _update_operand_images(self):
        # Update the annotations that has been updated
        for image in self.annotation_related_images:
            for d in self.csv['spec']['install']['spec']['deployments']:
                if d['name'] == image.deployment:
                    d['spec']['template']['metadata']['annotations'][self.RELATED_IMAGE_IDENTIFIER + image.name] = image.image

    def _setup_basic_logger(self):
        # Setup logging to stdout if we're not provided a logger
        self.log = logging.getLogger(__name__)
        out_hdlr = logging.StreamHandler(sys.stdout)
        out_hdlr.setFormatter(logging.Formatter('%(asctime)s %(levelname)s: %(message)s', '%Y-%m-%d %H:%M:%S'))
        out_hdlr.setLevel(logging.INFO)
        self.log.addHandler(out_hdlr)
        self.log.setLevel(logging.INFO)



class Image:
    def __init__(self, name=None, image=None, deployment=None, container=None):
        """Object to hold information about a container or related image

        :param image: Image
        :type image: string

        :param name: Name as identified in spec.relatedImages or olm.relatedImage annotation
        :type name: string

        :param deployment: Name of deployment(default: {None})
        :type deployment: string

        :param container: Name of container (default: {None})
        :type container: string
        """
        self.name       = name
        self.image      = image
        self.deployment = deployment
        self.container  = container
        self.tag        = None
        self.digest     = None

        # Everything before the last '/' should make up repo
        self.image_repo = '/'.join(image.split('/')[:-1])
        remainder = image.split('/')[-1]
        if '@' in remainder:
            self.image_name = remainder.split('@')[0]
            self.digest     = remainder.split('@')[1]
        elif ':' in remainder:
            self.image_name = remainder.split(':')[0]
            self.tag        = remainder.split(':')[1]
        else:
            self.image_name = remainder
            self.tag        = 'latest'

    def set_digest(self, digest):
        """Set image digest

        :param digest: Digest in format <type>:<hash>
        :type digest: string
        """
        if self.digest:
            # If already have a digest set we need to carefully replace
            self.image = self.image.replace(self.digest, digest)
            self.digest = digest
        else:
            self.digest = digest
            # We're assuming that when we're called the existing image doesn't have a digest
            self.image = '{}@{}'.format(self.image.split(':')[0], digest)
        return True

    def set_tag(self, tag):
        """Set new tag for image. Will update self.image only if image is identified by tag, but not overwrite digest

        :param tag: New tag to set
        :type tag: string
        """
        self.tag = tag
        if not self.digest:
            # We won't overwrite image info if we have digest
            self.image = '{}:{}'.format(self.image.split(':')[0], tag)

    def set_image_repo(self, repo):
        self.image      = self.image.replace(self.image_repo, repo)
        self.image_repo = repo
        return True
    
    def get_image_repo(self):
        """Returns the image_repo section of the overall image

        :return: Image repo i.e. quay.io/myrepo]
        :rtype: string
        """
        return self.image_repo

    def get_digest(self):
        """Returns image digest 

        :return: Digest in format <type>:<hash> 
        :rtype: string
        """
        if hasattr(self, 'digest'):
            return self.digest
        else:
            return None

    def get_olm_name(self):
        """Return the name of the image. Typically associated with olm.relatedImage and spec.relatedImages

        :return: Name
        :rtype: string
        """
        return self.name

    def get_image_name(self):
        """Returns the image name, stripped of repo and tag/digest

        :return: Name
        :rtype: string
        """
        return self.image_name

    def get_tag(self):
        """Get the tag associated with an image if known

        :return: Tag
        :rtype: string
        """
        if hasattr(self, 'tag'):
            return self.tag
        else:
            return None

    def get_image(self):
        """Return the full image 

        :return: Full image in format <repo>/<name>[@digest][:tag]
        :rtype: string
        """
        return self.image


from setuptools import setup, find_packages

setup(
    name='operator-csv-libs',
    version='1.8.5',
    description='Code to manage OLM related CSVs and channels',
    author='bennett-white',
    url='https://github.com/multicloud-ops/operator-csv-libs',
    packages=['operator_csv_libs'],
    install_requires=[
        'pyyaml',
        'pygithub',
        'dohq-artifactory',
        'slack_log_handler',
        'requests'
    ]
)